﻿using System;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Navigation;

namespace MyCalc01 {
  public sealed partial class MainPage : Page {
    private const string ButtonText = "789+C456-R123*M0±./=";
    private const int ButtonX = 5;
    private const int ButtonY = 4;
    private const int ButtonMargin = 10;
    private Button[] Button = new Button[ButtonX * ButtonY];
    private bool IsNewValue = true;
    private double Memory = 0;

    private Windows.Storage.StorageFolder SoundFolder;
    private Windows.Storage.StorageFile SoundFile;
    private Windows.Storage.Streams.IRandomAccessStream SoundStream;

    private enum Operations { None, Add, Subtract, Multiply, Divide };
    private Operations Operation = Operations.None;

    private double HiddenValue = 0.0;
    private double DisplayValueValue = 0.0;
    private double DisplayValue {
      get { return DisplayValueValue; }
      set {
        DisplayValueValue = value;
        if (!(TextBoxMain.Text.Contains(".") && TextBoxMain.Text.EndsWith("0")))
          try {
            TextBoxMain.Text = ((decimal)value).ToString();
          } catch (Exception) {
            TextBoxMain.Text = value.ToString();
          }
      }
    }

    public MainPage() {
      this.InitializeComponent();
    }

    protected async override void OnNavigatedTo(NavigationEventArgs e) {
      TextBoxMain.IsReadOnly = true;
      TextBoxMain.TextAlignment = TextAlignment.Right;
      TextBoxMain.Text = "0";

      GridButtons.ColumnDefinitions.Clear();
      GridButtons.RowDefinitions.Clear();
      GridButtons.Children.Clear();
       
      ColumnDefinition cd;
      RowDefinition rd;

      for (int i = 0; i < ButtonX; i++) {
        cd = new ColumnDefinition();
        cd.Width = new GridLength(100, GridUnitType.Auto);
        GridButtons.ColumnDefinitions.Add(cd);
      }

      for (int i = 0; i < ButtonY; i++) {
        rd = new RowDefinition();
        rd.Height = new GridLength(100, GridUnitType.Auto);
        GridButtons.RowDefinitions.Add(rd);
      }

      for (int i = 0; i < ButtonY; i++)
        for (int j = 0; j < ButtonX; j++) {
          int k = i * ButtonX + j;
          Button b = new Button();
          b.Background = new Windows.UI.Xaml.Media.SolidColorBrush(
            Windows.UI.Colors.CornflowerBlue);
          b.Foreground = new Windows.UI.Xaml.Media.SolidColorBrush(
            Windows.UI.Colors.Black);
          b.FontSize = 80;
          b.Content = ButtonText[k];
          b.SetValue(Grid.RowProperty, i);
          b.SetValue(Grid.ColumnProperty, j);
          b.Margin = new Thickness(ButtonMargin);
          b.Click += Button_Click;
          GridButtons.Children.Add(b);
          this.Button[k] = b;
        }
      SoundFolder = await Windows.ApplicationModel.Package.Current.
        InstalledLocation.GetFolderAsync("Sound");
      SoundFile = await SoundFolder.GetFileAsync("rim4416m.wav");
      SoundStream = await SoundFile.OpenAsync(Windows.Storage.FileAccessMode.Read);
    }

    private void Page_SizeChanged(object sender, SizeChangedEventArgs e) {
      int buttonWidth = (int)Window.Current.Bounds.Width / ButtonX - 
        ButtonMargin * 2;
      int buttonHeight = ((int)Window.Current.Bounds.Height - 200) / 
        ButtonY - ButtonMargin * 2;
      for (int i = 0; i < ButtonY; i++)
        for (int j = 0; j < ButtonX; j++) {
          int k = i * ButtonX + j;
          this.Button[k].Width = buttonWidth;
          this.Button[k].Height = buttonHeight; 
        }
    }

    void Button_Click(object sender, RoutedEventArgs e) {
      char c = ((Button)sender).Content.ToString()[0];
      switch (c) {
        case '+':
          UpdateValues();
          Operation = Operations.Add;
          break;
        case '-':
          UpdateValues();
          Operation = Operations.Subtract;
          break;
        case '*':
          UpdateValues();
          Operation = Operations.Multiply;
          break;
        case '/':
          UpdateValues();
          Operation = Operations.Divide;
          break;
        case '=':
          UpdateValues();
          Operation = Operations.None;
          break;
        case 'C':
          Initialize();
          Operation = Operations.None;
          break;
        case '±':
          if (!IsNewValue) {
            if (TextBoxMain.Text.StartsWith("-"))
              TextBoxMain.Text = TextBoxMain.Text.Substring(1);
            else
              TextBoxMain.Text = "-" + TextBoxMain.Text;
            SetDisplayValueFromText();
          }
          break;
        case '.':
          if (IsNewValue) TextBoxMain.Text = "0";
          if (!TextBoxMain.Text.Contains(".")) TextBoxMain.Text += c;
          IsNewValue = false;
          break;
        case '0':
        case '1':
        case '2':
        case '3':
        case '4':
        case '5':
        case '6':
        case '7':
        case '8':
        case '9':
          if (IsNewValue) TextBoxMain.Text = "";
          TextBoxMain.Text += c;
          SetDisplayValueFromText();
          IsNewValue = false;
          break;
        case 'M':
          Memory = DisplayValue;
          break;
        case 'R':
          DisplayValue = Memory;
          IsNewValue = false;
          break;
      }
      if (CheckBoxSound.IsChecked == true)
        MyMediaElement.SetSource(SoundStream, SoundFile.ContentType);
    }

    private void UpdateValues() {
      if (!IsNewValue) {
        switch (Operation) {
          case Operations.None:
            HiddenValue = DisplayValue;
            break;
          case Operations.Add:
            HiddenValue += DisplayValue;
            break;
          case Operations.Subtract:
            HiddenValue -= DisplayValue;
            break;
          case Operations.Multiply:
            HiddenValue *= DisplayValue;
            break;
          case Operations.Divide:
            HiddenValue /= DisplayValue;
            break;
        }
        IsNewValue = true;
        DisplayValue = HiddenValue;
      }
    }

    private void Initialize() {
      HiddenValue = 0.0;
      DisplayValue = 0.0;
      IsNewValue = true;
      Operation = Operations.None;
    }

    private void SetDisplayValueFromText() {
      DisplayValue = double.Parse(TextBoxMain.Text);
    }
  }
}
