﻿Public NotInheritable Class MainPage
  Inherits Page

  Private Const ButtonText As String =
    "789+C456-R123*M0±./="
  Private Const ButtonX As Integer = 5
  Private Const ButtonY As Integer = 4
  Private Const ButtonMargin As Integer = 10
  Private Button(ButtonX * ButtonY - 1) As Button
  Private IsNewValue As Boolean = True
  Private Memory As Double = 0.0

  Private SoundFolder As Windows.Storage.
    StorageFolder
  Private SoundFile As Windows.Storage.StorageFile
  Private SoundStream As Windows.Storage.Streams.
    IRandomAccessStream

  Private Enum Operations
    None
    Add
    Subtract
    Multiply
    Divide
  End Enum
  Private Operation As Operations = Operations.None

  Private HiddenValue As Double = 0.0
  Private DisplayValueValue As Double = 0.0
  Private Property DisplayValue As Double
    Get
      Return DisplayValueValue
    End Get
    Set(value As Double)
      DisplayValueValue = value
      If Not (TextBoxMain.Text.Contains(".") AndAlso
        TextBoxMain.Text.EndsWith("0")) Then
        Try
          TextBoxMain.Text = CDec(value).ToString()
        Catch ex As Exception
          TextBoxMain.Text = value.ToString()
        End Try
      End If
    End Set
  End Property

  Protected Overrides Async Sub OnNavigatedTo(
    e As Navigation.NavigationEventArgs)

    With TextBoxMain
      .IsReadOnly = True
      .TextAlignment = TextAlignment.Right
      .Text = "0"
    End With

    With GridButtons
      .ColumnDefinitions.Clear()
      .RowDefinitions.Clear()
      .Children.Clear()
    End With

    Dim cd As ColumnDefinition
    Dim rd As RowDefinition

    For i As Integer = 0 To ButtonX - 1
      cd = New ColumnDefinition()
      cd.Width = New GridLength(
        100, GridUnitType.Auto)
      GridButtons.ColumnDefinitions.Add(cd)
    Next

    For i As Integer = 0 To ButtonY - 1
      rd = New RowDefinition()
      rd.Height = New GridLength(
        100, GridUnitType.Auto)
      GridButtons.RowDefinitions.Add(rd)
    Next

    For i As Integer = 0 To ButtonY - 1
      For j As Integer = 0 To ButtonX - 1
        Dim k As Integer = i * ButtonX + j
        Me.Button(k) = New Button()
        With Me.Button(k)
          .Background = New SolidColorBrush(
            Windows.UI.Colors.CornflowerBlue)
          .Foreground = New SolidColorBrush(
            Windows.UI.Colors.Black)
          .FontSize = 80
          .Content = ButtonText(k)
          .SetValue(Grid.RowProperty, i)
          .SetValue(Grid.ColumnProperty, j)
          .Margin = New Thickness(ButtonMargin)
          AddHandler .Click, AddressOf Button_Click
        End With
        GridButtons.Children.Add(Me.Button(k))
      Next
    Next

    SoundFolder = Await Package.Current.
      InstalledLocation.GetFolderAsync("Sound")
    SoundFile = Await SoundFolder.GetFileAsync(
      "rim4416m.wav")
    SoundStream = Await SoundFile.OpenAsync(
      Windows.Storage.FileAccessMode.Read)
  End Sub

  Private Sub Page_SizeChanged(sender As Object,
    e As SizeChangedEventArgs)

    Dim buttonWidth As Integer =
      Window.Current.Bounds.Width \ ButtonX -
      ButtonMargin * 2
    Dim buttonHeight As Integer =
      (Window.Current.Bounds.Height - 200) \ ButtonY -
      ButtonMargin * 2

    For i As Integer = 0 To ButtonY - 1
      For j As Integer = 0 To ButtonX - 1
        Dim k As Integer = i * ButtonX + j
        Me.Button(k).Width = buttonWidth
        Me.Button(k).Height = buttonHeight
      Next
    Next
  End Sub

  Private Sub Button_Click(sender As Object,
    e As RoutedEventArgs)

    Dim c As Char = CType(sender, Button).Content
    Select Case c
      Case "+"c
        UpdateValues()
        Operation = Operations.Add
      Case "-"c
        UpdateValues()
        Operation = Operations.Subtract
      Case "*"c
        UpdateValues()
        Operation = Operations.Multiply
      Case "/"c
        UpdateValues()
        Operation = Operations.Divide
      Case "="c
        UpdateValues()
        Operation = Operations.None
      Case "C"c
        Initialize()
        UpdateValues()
      Case "±"c
        If Not IsNewValue Then
          If TextBoxMain.Text.StartsWith("-") Then
            TextBoxMain.Text =
              TextBoxMain.Text.Substring(1)
          Else
            TextBoxMain.Text = "-" &
              TextBoxMain.Text
          End If
          SetDisplayValueFromText()
        End If
      Case "."c
        If IsNewValue Then TextBoxMain.Text = "0"
        If Not TextBoxMain.Text.Contains(".") Then _
          TextBoxMain.Text &= c
        IsNewValue = False
      Case "0"c To "9"c
        If IsNewValue Then TextBoxMain.Text = ""
        TextBoxMain.Text &= c
        SetDisplayValueFromText()
        IsNewValue = False
      Case "M"c
        Memory = DisplayValue
      Case "R"c
        DisplayValue = Memory
        IsNewValue = False
    End Select

    If CheckBoxSound.IsChecked Then
      MyMediaElement.SetSource(SoundStream,
        SoundFile.ContentType)
    End If
  End Sub

  Private Sub UpdateValues()
    If Not IsNewValue Then
      Select Case Operation
        Case Operations.None
          HiddenValue = DisplayValue
        Case Operations.Add
          HiddenValue += DisplayValue
        Case Operations.Subtract
          HiddenValue -= DisplayValue
        Case Operations.Multiply
          HiddenValue *= DisplayValue
        Case Operations.Divide
          HiddenValue /= DisplayValue
      End Select
      IsNewValue = True
      DisplayValue = HiddenValue
    End If
  End Sub

  Private Sub Initialize()
    HiddenValue = 0.0
    DisplayValue = 0.0
    IsNewValue = True
    Operation = Operations.None
  End Sub

  Private Sub SetDisplayValueFromText()
    DisplayValue = Double.Parse(TextBoxMain.Text)
  End Sub
End Class
